%% Dbut du module simple_server
-module(simple_server).

-export([start/0, start/1]).

-define(default_port, 9999).

%% Dmarre le serveur sur le port par dfaut
start() ->
    start(?default_port).

start(Port) ->
    %% [1] Dmarre le serveur et rcupre la rfrence de la socket d'coute:
    SocketDEcoute = case gen_tcp:listen(Port, [{packet,0}, {active, false}]) of
                        %% L'accs au port demand est possible
                        {ok, LSock} -> io:format("Serveur dmarr sur le port ~p~n", [Port]),
                                       LSock;
                        %% Ou bien une erreur s'est produite:
                        {error, Raison} -> io:format("Erreur dans le dmarrage du serveur sur le port ~p: [~p]~n", 
                                                     [Port, Raison]),
                                           exit(Raison)
                   end,

    %% [2] Se place indfiniment en attente de connexion:
    {ok, Socket} = gen_tcp:accept(SocketDEcoute),

    %% [3] Reoit les donnes et les affiche
    {ok, Data} = do_recv(Socket, []),
    io:format("Le serveur a reu le message suivant: [~p]~n", [Data]),

    %% [4] Ferme la connexion vers la socket pour librer les ressources
    %% alloues.
    gen_tcp:close(Socket).

%% Il s'agit d'une boucle charge de recevoir des donnes jusqu' la
%% dconnexion du client.
%% La boucle rend la main  la fonction principale lorsque le client
%% se dconnecte, c'est--dire lorsque la lecture sur la socket de
%% connexion avec le client renvoie une erreur de type 'closed': La
%% socket est ferme suite  la dconnexion du client.
do_recv(Sock, Data) ->
    case gen_tcp:recv(Sock, 0) of
        {ok, NewData} ->
            do_recv(Sock, [Data, NewData]);
        %% Lorsque le client a ferm la connexion, on renvoie les
        %% donnes reues.
        {error, closed} ->
            {ok, lists:flatten(Data)}
    end.
%% Fin du module simple_server
