%%%-------------------------------------------------------------------
%%% File    : ldap_connections_sup.erl
%%% Author  : Mickael Remond <mremond@erlang.dyndns.org>
%%% Description : 
%%%
%%% Created : 28 Feb 2003 by Mickael Remond <mremond@erlang.dyndns.org>
%%%-------------------------------------------------------------------
-module(ldap_connections_sup).

-behaviour(supervisor).
%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
%%--------------------------------------------------------------------
-export([start_link/0, start_connection/1]).

%%--------------------------------------------------------------------
%% Internal exports
%%--------------------------------------------------------------------
-export([init/1]).

%%--------------------------------------------------------------------
%% Macros
%%--------------------------------------------------------------------
-define(SERVER, ?MODULE).

%%--------------------------------------------------------------------
%% Records
%%--------------------------------------------------------------------

%%====================================================================
%% External functions
%%====================================================================
%%--------------------------------------------------------------------
%% Function: start_link/0
%% Description: Starts the supervisor
%%--------------------------------------------------------------------
start_link() ->
    supervisor:start_link({local, ?SERVER}, ?MODULE, []).

%% Handle a new client connection:
start_connection(Socket) ->
    io:format("Lancement d'une nouvelle connexion:~p~n", [Socket]),
    supervisor:start_child(?SERVER, [Socket]).

%%====================================================================
%% Server functions
%%====================================================================
%%--------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%--------------------------------------------------------------------
init([]) ->
    AChild = {ldap_connection_srv,
	      {ldap_connection_srv, start_link, []}, transient, 5000,
	      worker, [ldap_connection_srv,'ELDAPv3']},
    {ok,{{simple_one_for_one,0,1}, [AChild]}}.

%%====================================================================
%% Internal functions
%%====================================================================
