%%% File    : ldap_handler_file.erl
%%% Author  : Mickael Remond <mremond@erlang.dyndns.org>
%%% Description : 
%%% Created :  8 Mar 2003 by Mickael Remond <mremond@erlang.dyndns.org>

-module(ldap_handler_file).
-author("mickael.remond@erlang-fr.org").

%% Plugin API
-export([start/1, stop/1, log/5]).

-define(app_name, ldap_proxy).

%% Initialize plugin
start(ExtraArgs) ->
   case lists:keysearch(filename, 1, ExtraArgs) of
       false -> io:format("Plugin error: Error initializing plugin ldap_handler_file.~n", []),
		ok;
       {value, {filename, Filename}} ->
	   %% Open the file
	   {ok, FD} = file:open(Filename, [append,binary]),
	   application:set_env(?app_name, file_descriptor, FD),
	   ok
   end.

%% Terminate plugin
stop(ExtraArgs) ->
    %% Get file descriptor. This is not handled in the ldap_proxy_config module because
    %% this is plugin dependant
    case application:get_env(?app_name, file_descriptor) of
	{ok, FD} -> file:close(FD);
	_Other   -> file_was_not_open
    end.

%% Plugin main hook
log(ServerSocket, ClientSocket, Side, Data, ExtraArgs) ->
    %% Get file descriptor. This is not handled in the ldap_proxy_config module because
    %% this is plugin dependant
    case application:get_env(?app_name, file_descriptor) of
	{ok, FD} -> Msg = io_lib:format("********~nSocket: ~p~n-> ~s data: ~p ~n", [ClientSocket, Side, Data]),
                    Result = file:write(FD, Msg);
	_Other   -> io:format("Plugin error: Cannot process LDAP information.~n", [])
    end.
