%%%-------------------------------------------------------------------
%%% File    : ldap_proxy_sup.erl
%%% Author  : Mickael Remond <mremond@mremond.cgey.ifp.fr>
%%% Description : Main supervisor for the ldap proxy application.
%%%
%%% Created :  5 Mar 2003 by Mickael Remond <mremond@mremond.cgey.ifp.fr>
%%%-------------------------------------------------------------------
-module(ldap_proxy_sup).

-behaviour(supervisor).
%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
%%--------------------------------------------------------------------
-export([
	 start_link/1
        ]).

%%--------------------------------------------------------------------
%% Internal exports
%%--------------------------------------------------------------------
-export([
	 init/1
        ]).

%%--------------------------------------------------------------------
%% Macros
%%--------------------------------------------------------------------
-define(SERVER, ?MODULE).

%%--------------------------------------------------------------------
%% Records
%%--------------------------------------------------------------------

%%====================================================================
%% External functions
%%====================================================================
%%--------------------------------------------------------------------
%% Function: start_link/1
%% Description: Starts the supervisor
%%--------------------------------------------------------------------
start_link(_StartArgs) ->
    supervisor:start_link({local, ?SERVER}, ?MODULE, []).

%%====================================================================
%% Server functions
%%====================================================================
%%--------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%--------------------------------------------------------------------
init(Args) ->
    %% Start two child: The acceptor and the connection supervisor.
    Acceptor = {ldap_acceptor,{ldap_accept_fsm,start_link,[]},
		permanent,5000,worker,[ldap_accept_fsm]},
    Connection_sup = {ldap_connections, {ldap_connections_sup, start_link, []},
		      permanent, 5000, supervisor, [ldap_connection_sup]},
    {ok,{{one_for_all,2,5}, [Acceptor, Connection_sup]}}.

    %% TODO:
    %% In a future version I will provide a way to have separate
    %% process to share ldap connections with several client (as long
    %% as they use the same binding rights

%%====================================================================
%% Internal functions
%%====================================================================
