-module(cellule).
-export([create/2, create/3, loop/7]).

%% Frquence des gnrations = 1 seconde.
-define(frequence, 2).

%% TODO: Faire un module helper permettant de manier les coordonnes ?

%% L'tat d'une cellule est dtermine par son statut.  Une cellule
%% dispose de coordonne et d'une gnration et d'un statut.

%% A partir d'une coordonne, elle envoie son statut (morte ou
%% vivante) en broadcast et se met en attente de rception de des
%% messages (changement d'tat) Lorsque toutes ses informations sont
%% reues, elle peut  nouveau changer d'tat et envoi les
%% informations de gnration suivante.

%% La tche la plus dlicate consiste  dterminer les cellules qui
%% dont des informations sont attendues  partir des coordonnes de la
%% cellule et de la taille de la matrice. C'est la premire fonction 
%% dmontrer
%% Coordonnes d'une cellule: {Horizontale, Verticale}. 1,1 = Haut  gauche
%% Illustration classique de la manire dont on opre des tests en Erlang:
%% Utilisation du pattern matching
%% L'ordre des squences est important: Si les bords sont dfinis avant les
%% coins (cas le plus particulier, les coins ne peuvent tre trait correctement
%% puisque la correspondance s'effectuera en premier sur le bord.
%% TODO: Faut-il inverser Matrice et coordonnes dans la signature de 
%%       la cellule ?
%% Les coins constituent 4 cas particuliers qui n'ont que 2 voisins.
%% Cas particulier n1: La cellule est dans le coin en haut  gauche
voisin(Coordonnes = {1,1}, Matrice = {I,J}) ->
    [{1,2}, {2,1}];
%% Cas particulier n2: La cellule est dans le coin en haut  droite
voisin(Coordonnes = {I,1}, Matrice = {I,J}) ->
    [{I-1, 1}, {I, 2}];
%% Cas particulier n3: La cellule est dans le coin en bas  gauche
voisin(Coordonnes = {1,J}, Matrice = {I,J}) ->
    [{1, J - 1},{2, J}];
%% Cas particulier n4: La cellule est dans le coin en bas  droite
voisin(Coordonnes = {I,J}, Matrice = {I,J}) ->
    [{I, J - 1},{I - 1, J}];
%% Les bords n'ont que 3 voisins.
%% Bord haut: Y diffrent de J => Ce n'est pas cas particulier 3
voisin(Coordonnes = {1,Y}, Matrice = {I,J}) ->
    [{1, Y - 1},{1, Y + 1}, {2, Y}];    
%% Bord gauche: X different de I => Ce n'est pas le cas particulier 4
voisin(Coordonnes = {X,1}, Matrice = {I,J}) ->
    [{X - 1, 1},{X + 1, 1}, {X, 2}];
%% Bord bas: X different de I => Ce n'est pas le cas particulier 4
voisin(Coordonnes = {X,J}, Matrice = {I,J}) ->
    [{X - 1, J},{X + 1, 1}, {X, 2}];
%% Bord droite: Y different de J => Ce n'est pas le cas particulier 4
voisin(Coordonnes = {I,Y}, Matrice = {I,J}) ->
    [{I - 1, Y},{I, Y -1}, {I, Y + 1}];
%% Le cas gnral: 4 voisins
voisin(Coordonnes = {X,Y}, Matrice = {I,J}) ->
    [{X, Y - 1}, {X, Y + 1}, {X - 1, Y}, {X + 1, Y}].

%% Si aucun tat n'est fournit lors de la cration de la cellule, on
%% propose un tat alatoire
create(Coordonnes, Matrice) ->
    Etat = case random:uniform(2) of
	       1 -> alive;
	       2 -> dead
	   end,
    create(Coordonnes, Etat, Matrice).

create(Coordonnes = {X, Y}, Etat, Matrice = {I, J}) ->
    Generation = 0,
    Voisins = voisin(Coordonnes, Matrice),
    PidDispatcher = self(),
    CellulePid = spawn(?MODULE, loop, [Coordonnes, Etat, Generation, PidDispatcher, Voisins, [], length(Voisins)]),
    PidDispatcher ! message(Coordonnes, Generation,  Etat),
    CellulePid.

%% Cas particulier, on dispose des informations de toutes les cellules de mme gnration
loop(Coordonnes, Etat, Generation, PidDispatcher, Voisins, TamponMessages, 0) ->
    %% Calcule le nouvel tat: 0 voisins: meurt, 1,2: Voisin vit, 3, 4 voisin meurt
    %% On peut placer un receive ici pour dclencher le passage  la prochaine
    %% Itration, par exemple depuis l'interface utilisateur.
    timer:sleep(?frequence * 1000),

    NewEtat = prochain_etat(Etat, TamponMessages),
    PidDispatcher ! message(Coordonnes, Generation + 1, NewEtat),

    loop(Coordonnes, NewEtat, Generation + 1, PidDispatcher, Voisins, [], length(Voisins));
%% Cas gnral, on reoit les messages des cellules de mme gnration
loop(Coordonnes, Etat, Generation, PidDispatcher, Voisins, TamponMessages, VoisinsManquants) ->
    %% Ne reoit que les messages en provenance d'une gnration donne.
    receive
	{cellule, CoordCell, Generation, EtatCellule} ->
	    case lists:member(CoordCell, Voisins) of
		%% Stocke les informations des cellules voisines dans le tampon
		true -> loop(Coordonnes, Etat, Generation, PidDispatcher, Voisins, [EtatCellule|TamponMessages], VoisinsManquants - 1);
		%% Si le message ne provient pas d'un voisin, on l'ignore
		false -> loop(Coordonnes, Etat, Generation, PidDispatcher, Voisins, TamponMessages, VoisinsManquants)
	    end
    end.
    
message(Coordonnes, Generation, Etat) ->
    {cellule, Coordonnes, Generation, Etat}.

%% TODO: A modliser  partir des vrais rgles
%% Calcule le nouvel tat: 0 voisins: meurt, 1,2: Voisin vit, 3, 4 voisin meurt
prochain_etat(Etat, EtatsVoisin) ->
    NbVoisinsVivants = count(alive, EtatsVoisin),
    case NbVoisinsVivants of
	0 -> dead;	    
	1 -> alive;
	2 -> alive;
	3 -> dead;
	4 -> dead
    end.

count(Etat, EtatsVoisin) ->
    count(Etat, EtatsVoisin, 0).
count(Etat, [], Nb) ->
    Nb;
count(Etat, [Etat|AutresEtats], Nb) ->
    count(Etat, AutresEtats, Nb + 1);
count(Etat, [EtatVois|EtatsVois], Nb) ->
    count(Etat, EtatsVois, Nb).    
    

%% Dans la partie sur l'interface TK, faire une interface graphique
%% pour notre jeu de la vie, pour illustrer la cration d'interface
%% graphique
%% TODO: L'affichage du jeu se fait par cration d'une cellule
%% d'observation simplement charge de surveiller le jeu (dans un
%% module  part).

%% TODO: Cet exemple peut faire l'objet d'un cas pratique avec de multiple dclinaisons. La premire est que cette version contient une fuite mmoire. Montrer comment le dbogueur peut nous aider  la trouver
%% On peut le trouver rien qu'avec la fonction procinfo du mode d'arrt ctrl-C. On constate que la file d'attente de message croit sans cesse et conserve des message concernant des gnrations de processus antrieurs: Pour corriger le bug, il faut vider la file d'attente des messages des messages de la gnration actuelle avant de passer  la suivante.


%% TODO: Ajout de l'interface graphique.
%% TODO: Modification du module pour qu'il devienne un jeu de la vie distribu.
%% TODO: Possibilit d'introduction d'autres critres comme l'age de la cellule, qui galement dterminer son tat (age = dure durant laquelle la cellule est vivante)
%% TODO: Raffiner et ajouter des contrles pour pouvoir paramtrer soit mme le systme: La modification met le systme en pause. Le redmarrage recre les cellules avec une gnration 0.

%% Montrer comment on peut ajuster dynamiquement la frquence.
