-module(game).
-compile(export_all).

start(Matrice) ->
    %% Initialise le gnrateur de nombre altoire
    {A,B,C} = now(),
    random:seed(A,B,C),
    %% Dmarre le programme avec une interface graphique par dfaut
    start(texte_ui, Matrice).

start(UIModuleHandler, Matrice = {X,Y}) ->
    %% Gnration de la liste des cellules de la matrice par list
    %% comprehension
    ToutesCellules = [ {I,J} || I <- lists:seq(1, X), J <- lists:seq(1, Y)],

    %% Creation de tous les processus cellules et rcupration de la
    %% liste des Pids de toutes les cellules.
    CellulePids = lists:map(fun(Coord) -> cellule:create(Coord, Matrice) end, ToutesCellules),

    %% Dmarrage de l'interface utilisateur
    UIPid = spawn(UIModuleHandler, start, [length(CellulePids), 0, Matrice]),

    %% Appel de la fonction de dispatch.
    dispatch([UIPid|CellulePids]).

%% Broadcast
%% Renvoie les informations des cellules vers toutes les cellules et
%% vers le processus d'interface utilisateur
dispatch(Pids) ->
    receive
	Message ->
	    lists:foreach(fun(Pid) ->
				  Pid ! Message end, Pids),
	    dispatch(Pids)
    end.
    
