%% TODO: Create later on a persistence package.
-module(mnesia_rei).

-export([create/0]).                     %% Database management
-export([check_user/2, create_user/2]).  %% User management
-export([new_object/3, list_objects/0,   %% World management
	 get_object/1, update_object/3]).

%% Database schema
-include("rei.hrl").

%% Database schema creation (initialisation)
create() ->
    Nodes = [node()],
    mnesia:create_schema(Nodes),
    mnesia:start(),
    %% The user table is only stored in memory.
    %% This means that the user list is resetted for the moment.
    mnesia:create_table(user, [{attributes, record_info(fields, user)},
                              {type, set}, {ram_copies, Nodes}]),
    mnesia:create_table(object, [{attributes, record_info(fields, object)},
                                 {type, set}, {disc_copies, Nodes}]),
    mnesia:info(),
    mnesia:stop().

%% ** User management
%% check if user exist and if password is correct
%% Returns: ok if user exist and password is ok
%%          incorrect_password if user exist and password is incorrect
%%          incorrect_user if user does not exist
check_user(Login, PasswordTry) ->
    Read = fun() ->
		   mnesia:read({user, Login}) end,
    case mnesia:transaction(Read) of
	{atomic,[]} ->
	    incorrect_user;
	{atomic, [#user{login = Login, password=Password}]} -> 
            
	    check_password(PasswordTry, Password)
    end.

check_password(Password, Password) ->	    
    ok;
check_password(Try, Password) ->
    incorrect_password.

%% Insert new user into the database
create_user(Login, Password) ->
    Insert = fun() ->
                     mnesia:write(#user{login=Login, password=Password}) end,
    mnesia:transaction(Insert),
    %% Create the actor object with default position
    new_object( "mainactor_" ++ Login, snowmanclass, {128.0, 0.0, 120.0} ).

%% ** World object management
%% REI World object creation
new_object(Id, Class, Pos) ->
    Insert = fun() ->
                     mnesia:write(#object{id=Id, class=Class, pos=Pos}) end,
    mnesia:transaction(Insert).

%% REI World object creation
update_object(Id, Class, Pos) ->
    Insert = fun() ->
                     mnesia:write(#object{id=Id, class=Class, pos=Pos}) end,
    mnesia:transaction(Insert).

%% List objects in the world:
%% Send back all the objects id
list_objects() ->
    Get_keys = fun() ->
		       mnesia:all_keys(object) end,
    {atomic, Keys} = mnesia:transaction(Get_keys),
    Keys.

%% Get an object according with its ID (Key)
get_object(Id) ->
    Get_object = fun() ->
			 mnesia:read({object, Id}) end,
    {atomic, ObjectList} = mnesia:transaction(Get_object),
    case ObjectList of
    	[] -> nothing ;
	[Object] -> Object
    end.

%% Check if the login is correct:
%% Return either: ok, unknown_user, incorrect password
%%   check_user(User, Password) ->

%% I keep this code handy for later development
%list_objects() ->
%    F = fun() ->
%        Female = #employee{sex = female, name = '$1', _ = '_'},
%        mnesia:select(employee, [{Female, [], ['$1']}])
%    mnesia:transaction(F).
