-module(rei_lib).

-export([parse_parameters/1, get_value/3,
        notify_message/3,
	f2l/1]).

%% Parse the parameters of a REI ip protocol command
%% The input format is a string of the form:
%%  login=mikl password=123
%% This function transform this into a list of tuples of length 2.
parse_parameters(Params) ->
    F = fun(X) ->
               [Key, Value] = string:tokens(X, [$=]),
               {httpd_util:to_lower(Key), httpd_util:to_lower(Value)} end,
    lists:map(F, Params).

%% Read the value associated with a given parameter
%% and launch conversion to the proper type
get_value(Params, Key, Type) ->
    Result = case lists:keysearch(Key, 1, Params) of
                 {value, {Key, Value}} -> Value;
                 false -> undefined
             end,
    convert(Type, Result).

convert(pos, Value) when list(Value) ->
    [X,Y,Z] = string:tokens(Value, [${ , $, , $}]),
    {to_num(X), to_num(Y), to_num(Z)};
convert(pos, Value) ->
    {0.0, 0.0, 0.0};
convert(string, Value) when atom(Value) ->
    atom_to_list(Value);
convert(string, Value) ->    
    lists:flatten(Value);
convert(atom, Value) when atom(Value) ->
    Value;
convert(atom, Value) when list(Value) ->
    list_to_atom(Value).

%% TODO: Should we handle conversion if we passed an integer as
%% parameters ?
to_num(Value) ->
    list_to_float(Value).

%% Compose a notify message:
notify_message(Id, Class, Pos) ->
    {X, Y, Z} = Pos,
    Message = io_lib:format("INFO object id=~s class=~p pos=~f,~f,~f\r\n",
                            [Id,
                             Class,
                             X, Y, Z]).

%% Float to list conversion
%% never use scientific notation
f2l(Float) ->
    lists:flatten(io_lib:format("~f", [Float])).
