%%%-------------------------------------------------------------------
%%% File    : rei_server_connect_sup.erl
%%% Author  : Mickael Remond <mickael.remond@erlang-fr.org>
%%% Description : Supervisor for dynamic child handling client connections
%%%
%%% Created : 24 Nov 2002 by Mickael Remond <mickael.remond@erlang-fr.org>
%%%-------------------------------------------------------------------
-module(rei_server_connect_sup).

-behaviour(supervisor).
%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
%%--------------------------------------------------------------------
-export([start_link/0]).

%%--------------------------------------------------------------------
%% Internal exports
%%--------------------------------------------------------------------
-export([init/1]).

%%--------------------------------------------------------------------
%% Macros
%%--------------------------------------------------------------------
-define(SERVER, ?MODULE).

%%--------------------------------------------------------------------
%% Records
%%--------------------------------------------------------------------

%%====================================================================
%% External functions
%%====================================================================
%%--------------------------------------------------------------------
%% Function: start_link/0
%% Description: Starts the supervisor
%%--------------------------------------------------------------------
start_link() ->
    supervisor:start_link({local, ?SERVER}, ?MODULE, []).

%%====================================================================
%% Server functions
%%====================================================================
%%--------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%--------------------------------------------------------------------
init([]) ->
    DispatchSrv = rei_server_dispatch_srv,
    {ok,{{simple_one_for_one,5,5}, [{DispatchSrv, {DispatchSrv, start_link, []}, transient, 2000, worker, [DispatchSrv]}]}}.

%% Be carefull: simple_one_for_one implies that we pass the child
%% spec => We need to add synchronisation to be able to transfer the
%% sock. The socket should be transmitted as a parameter of the start_child...


%%====================================================================
%% Internal functions
%%====================================================================
