%%%-------------------------------------------------------------------
%%% File    : rei_server_notify_srv.erl
%%% Author  : Mickael Remond <mremond@erlang.dyndns.org>
%%% Description : 
%%%
%%% Created : 18 Mar 2003 by Mickael Remond <mremond@erlang.dyndns.org>
%%%-------------------------------------------------------------------
-module(rei_server_notify_srv).
-author("mickael.remond@erlang-fr.org").

-behaviour(gen_server).
%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
-export([start_link/0, add/1, remove/1, notify/2]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-record(state, {client_dispatch_pids=[]}).

%%====================================================================
%% External functions
%%====================================================================
%%--------------------------------------------------------------------
%% Function: start_link/0
%% Description: Starts the server
%%--------------------------------------------------------------------
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

%% Add a client in the distribution list
add(Pid) ->
    gen_server:cast(?MODULE, {add, Pid}).

remove(Pid) ->
    gen_server:cast(?MODULE, {remove, Pid}).

%% send a message to all clients
notify(FromPid, Message) ->
    gen_server:cast(?MODULE, {notify, FromPid, Message}).
    
%%====================================================================
%% Server functions
%%====================================================================

%%--------------------------------------------------------------------
%% Function: init/1
%% Description: Initiates the server
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%%--------------------------------------------------------------------
init([]) ->
    {ok, #state{}}.


%%--------------------------------------------------------------------
%% Function: handle_cast/2
%% Description: Handling cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%%--------------------------------------------------------------------
%% TODO: Pour faciliter la maintenance des processus on va attraper la fin des processus concerns en 
%% en se liant  eux, en trappant l'exit et en faisant un remove de la liste lorsque l'un deux est mort
%% Cette tche est hybride entre superviseur et travailleur
%% Mail: mailing list erlang - Faut-il imaginer un comportement
%% superviseur avec tat, capable de recevoir des messages et de les
%% traiter ?
handle_cast({add, Pid}, State) ->
    Pids = State#state.client_dispatch_pids,
    {noreply, State#state{client_dispatch_pids = Pids ++ [Pid]}};
handle_cast({remove, Pid}, State) ->
    Pids = State#state.client_dispatch_pids,
    {noreply, State#state{client_dispatch_pids = Pids -- [Pid]}};
handle_cast({notify, FromPid, Message}, State) ->
    AllPids = State#state.client_dispatch_pids,
    %% Removing the socket that does not need the message from the list
    NotifiedPids = lists:subtract(AllPids, [FromPid]),
    lists:foreach(fun(Pid) ->
			  rei_server_dispatch_srv:send(Pid, Message)
		  end,
		  NotifiedPids),
    {noreply, State};
handle_cast(Msg, State) ->
    {noreply, State}.

%% Unused gen_server callbacks
handle_call(Request, From, State) ->
    Reply = ok,
    {reply, Reply, State}.
handle_info(Info, State) ->
    {noreply, State}.
terminate(Reason, State) ->
    ok.
code_change(OldVsn, State, Extra) ->
    {ok, State}.

%%--------------------------------------------------------------------
%%% Internal functions
%%--------------------------------------------------------------------
