%%%-------------------------------------------------------------------
%%% File    : rei_server_sup.erl
%%% Author  : Mickael Remond <mickael.remond@erlang-fr.org>
%%% Description : Main Rei Server supervisor
%%%
%%% Created : 23 Nov 2002 by Mickael Remond <mickael.remond@erlang-fr.org>
%%%-------------------------------------------------------------------
-module(rei_server_sup).

-behaviour(supervisor).
%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
%%--------------------------------------------------------------------
-export([start_link/1]).

%%--------------------------------------------------------------------
%% Internal exports
%%--------------------------------------------------------------------
-export([init/1]).

%%--------------------------------------------------------------------
%% Macros
%%--------------------------------------------------------------------
-define(SERVER, ?MODULE).

%%--------------------------------------------------------------------
%% Records
%%--------------------------------------------------------------------

%%====================================================================
%% External functions
%%====================================================================
%%--------------------------------------------------------------------
%% Function: start_link/1
%% Description: Starts the supervisor
%%--------------------------------------------------------------------
start_link(_StartArgs) ->
    supervisor:start_link({local, ?SERVER}, ?MODULE, []).

%%====================================================================
%% Server functions
%%====================================================================
%%--------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%--------------------------------------------------------------------
init([]) ->
    AChild = {'rei_server_accept_fsm',{'rei_server_accept_fsm',start_link,[]},
	      permanent,2000,worker,['rei_server_accept_fsm']},
    NotifySrv = {'rei_server_notify_srv',{'rei_server_notify_srv',start_link,[]},
	      permanent,2000,worker,['rei_server_notify_srv']},
    CChild = {'rei_server_connect_sup', {'rei_server_connect_sup', start_link, []},
              permanent, 2000, supervisor, ['rei_server_connect_sup']},
    {ok,{{one_for_all,0,1}, [AChild, NotifySrv, CChild]}}.

%%====================================================================
%% Internal functions
%%====================================================================
