-module(serveur_sup2).
-export([start/0, heure/0, erreur/0, serveur/1, supervisor/0, stop/0]).

-define(nom_serveur, time_server).

%% Fonction cliente du serveur:
%% Lance le serveur
start() ->
    %% TODO: Register le processus globalement avec le nom time_server
    spawn(?MODULE, supervisor, []).

%% Fonction cliente du serveur:
%%  Renvoie leur et le nombre de requte dj servies
heure() ->
    %% Envoi la requte au processus serveur
    server_call(?nom_serveur, {time, self()}),

    %% Rcupre la rponse du serveur
    receive
	Answer -> Answer
    after 30000 ->
	    {timeout}
    end.

%% Fonction client permettant de stopper le serveur
%% La sortie du processus serveur est normal:
%% Le superviseur est programmer pour s'arrter galement dans cette hypothse
stop() ->
    %% Envoi la requte au processus serveur
    server_call(?nom_serveur, {stop}).

%% Autre fonction client du serveur:
%% Cette requte gnre une erreur.
erreur() ->
    %% Envoi la requte au processus serveur
    server_call(?nom_serveur, {erreur}).

%% ----
%% Fonction de factorisation d'envoi des message au serveur
%% Gre l'envoi du message au serveur nomme
%% (Utilise par les fonctions clientes
server_call(NomServeur, Message) ->
    %% Recupre l'identifiant de processus du serveur
    PidServer = global:whereis_name(NomServeur),

    %% Envoi la requte au processus serveur, s'il existe
    case PidServer of
	undefined -> error;
	Pid -> Pid ! Message
    end.

%% ----
%% Fonction supervisor/0 dmarrant le superviseur et le processur
%% travailleur supervis
supervisor() ->
    %% Caractristique d'un processus superviseur
    process_flag(trap_exit,true),
    process_flag(save_calls, 1),
    %% Lance le serveur en le liant avec le processus superviseur
    Pid = spawn_link(?MODULE, serveur, [0]),
    global:register_name(?nom_serveur, Pid),
    supervise(Pid).

%% ----
%% fonction supervise/1 implmentant la boucle du superviseur
supervise(Pid) ->
    receive 
	%% Si le processus serveur se termine normalement, le
	%% processus superviseur se termine galement
	{'EXIT', Pid, normal} ->
	    ok;
	%% Si le processus superviseur se termine en erreur, on
	%% analyse l'erreur pour rcuprer l'tat du serveur et on
	%% relance le processus.
	{'EXIT', Pid, Error} ->
	    %% TODO: Rcupration de l'tat du server
	    Info = process_info(Pid, last_calls),
	    io:format("Erreur dans le serveur. Redmarrage~n [~p]~n",[Info]),
	    NewPid = spawn_link(?MODULE, serveur, [0]),
	    global:register_name(?nom_serveur, NewPid),
	    supervise(NewPid);
	%% Si d'autres types de messages sont reu ou bien des
	%% messages d'erreurs en provenance d'autres processus, on ne
	%% fait rien, mais on maitien le processus superviseur
	Other ->
	    supervise(Pid)
    end.

%% ----
%% Fonction implmentant la boucle du serveur
serveur(NombreRequetes) ->
    receive
	{time, Pid} ->
	    %% Envoi la rponse au client
	    Pid ! {erlang:now(), NombreRequetes},
	    %% Boucle serveur
	    serveur(NombreRequetes + 1);
	{stop} ->
	    %% Arrte le serveur (sortie normal de la boucle)
	    ok;
	{erreur} ->
	    %% Attention: Gnre une erreur d'excution de type badmatch
	    1 = 2;
	%% Il n'y a pas besoin de boucler pour raliser un serveur tolrant au panne:
	%% La boucle serveur est relance par le superviseur.
	Autre ->
	    %% Ne fait rien lorsqu'un message inconnu est reu: Boucle
	    %% sur le serveur sans incrmenter le nombre de requtes.
	    serveur(NombreRequetes)
    end.
    
	    
    
