%% Dbut du module erreur_retour
-module(erreur_retour).
-export([start/0]).

%% Programme simulant la gestion de donnes utilisateur
start() ->
    Donnees = genere_donnees(),
    dialogue_sauvegarde(Donnees).

%% Implmente le dialogue avec l'utilisateur pour la gestion des
%% sauvegardes
dialogue_sauvegarde(Donnees) ->
    NomFichier = demande_nom_fichier(),
    case ecrit_fichier(NomFichier, Donnees) of
        ok -> 
            ok;
        {error, {TypeErreur, {Code, Commentaire}}} -> 
            io:format("Erreur de sauvegarde ~p ~p (~p)~n-> Recommencez~n~n",
                      [TypeErreur, Commentaire, Code]),
            dialogue_sauvegarde(Donnees)
    end.

%% Sauvegarde des donnes dans le fichier
%% La gestion des trois tapes (ouverture, criture et fermeture du
%% fichier est gr par la fonction file:write_file/2
%% La fonction renvoie ok ou {error, Raison}
ecrit_fichier(NomFichier, Donnees) ->
    case file:write_file(NomFichier, list_to_binary(Donnees)) of
        ok ->
            ok;
        {error, Raison} ->
            {error, {'Fichier', {Raison, "Impossible de sauvegarder les donnes"}}}
    end.

%%genere_donnees() ->
%% ...
%%demande_nom_fichier() ->    
%% ...
%% Les fonction genere_donnees et demande_nom_fichier sont identiques
%%  celle du module throw_ex

%% Gnre alatoirement 1000 caractres imprimables
genere_donnees() ->
    %% Initialise le gnrateur de nombre alatoire:
    {N1, N2, N3} = now(),
    random:seed(N1,N2,N3),

    genere_donnees(1000, []).

genere_donnees(0, Acc) ->
    Acc;
genere_donnees(NbCaractere, Acc) ->
    Caractere = random:uniform(90) + 32,
    genere_donnees(NbCaractere - 1, [Caractere | Acc]).

%% Demande le nom du fichier de sauvegarde  l'utilisateur
demande_nom_fichier() ->    
    io:format("** Sauvegarde **~n", []),
    NomFichier = io:get_line('Nom du fichier: '),
    string:strip(NomFichier, right, 10).  % Supprime le retour chariot
                                          % (code ascii 10)  la fin
                                          % du nom de fichier

%% Fin du module erreur_retour

