%% Dbut du module throw_ex
-module(throw_ex).
-export([start/0]).

%% Programme simulant la gestion de donnes utilisateur
start() ->
    Donnees = genere_donnees(),
    dialogue_sauvegarde(Donnees).

%% Implmente le dialogue avec l'utilisateur pour la gestion des
%% sauvegardes
dialogue_sauvegarde(Donnees) ->
    case catch sauve(Donnees) of
        ok -> 
            ok;
        {TypeErreur, {Code, Commentaire}} -> 
            io:format("Erreur de sauvegarde ~p ~p (~p)~n-> Recommencez~n~n",
                      [TypeErreur, Commentaire, Code]),
            dialogue_sauvegarde(Donnees)
    end.

%% Fonction de sauvegarde des donnes manipules dans le programme
sauve(Donnees) ->
    NomFichier = demande_nom_fichier(),
    ecrit_fichier(NomFichier, Donnees).

%% Gnre alatoirement 1000 caractres imprimables
genere_donnees() ->
    %% Initialise le gnrateur de nombre alatoire:
    {N1, N2, N3} = now(),
    random:seed(N1,N2,N3),

    genere_donnees(1000, []).

genere_donnees(0, Acc) ->
    Acc;
genere_donnees(NbCaractere, Acc) ->
    Caractere = random:uniform(90) + 32,
    genere_donnees(NbCaractere - 1, [Caractere | Acc]).

%% Demande le nom du fichier de sauvegarde  l'utilisateur
demande_nom_fichier() ->    
    io:format("** Sauvegarde **~n", []),
    NomFichier = io:get_line('Nom du fichier: '),
    string:strip(NomFichier, right, 10).  % Supprime le retour chariot
                                          % (code ascii 10)  la fin
                                          % du nom de fichier

%% Sauvegarde des donnes dans le fichier
%% Lve une exception si quelque chose se passe mal dans l'criture
%% (Impossible d'ouvrir le fichier en criture, manque de place sur le
%% disque, etc.
ecrit_fichier(NomFichier, Donnees) ->
    IO = case file:open(NomFichier, [write]) of % Ouverture du fichier en
                                                % criture
             {ok, IODevice} -> IODevice;
             {error, Raison1} -> throw({'Fichier', {Raison1, "Impossible d'ouvrir le fichier en criture"}})
         end,

    case file:write(IO, Donnees) of % Ecriture des donnes
        ok -> ok;
        {error, Raison2} -> throw({'Fichier', {Raison2, "Impossible d'crire dans le fichier"}})
    end,

    case file:close(IO) of % Fermeture du fichier
        ok -> ok;
        {error, Raison3} -> throw({'Fichier', {Raison3, "Impossible de fermer le fichier"}})
    end,
    ok.
%% Fin du module throw_ex
