-module(helloserver).

%% fonctions de gestion du serveur
-export([start/0, stop/0]).

%% fonctions de stockage et rcupration des valeurs
-export([store/2, retrieve/1]).

%% La fonction serveur elle-mme (boucle).
-export([loop/1]).

%% Le processus du serveur se verra dot d'un nom, enregistr sur 
%% le noeud sur lequel tourne le processus.
-define(server, hellosrv).

%% ----------------------------------------
%% Interface du serveur (API)

%% start: Dmarrage du serveur.
%% Donne un nom au processus serveur
start() ->
    Etat = [],
    Pid = spawn(?MODULE, loop, [Etat]),
    register(?server, Pid).

%% Stop: arrte le serveur ( partir de son nom)
stop() ->
    ?server ! {stop, self()}.

%% enregistre
%%  Pid = Identifiant de processus
%%  Cle = chane de caractres
%%  Valeur = chane de caractres
%% Renvoie: ok
store(Cle, Valeur) ->
    Self = self(),
    ?server ! {store, Self, Cle, Valeur},
    receive
	{helloserver, Self, Cle} ->
	    ok
    end.

%% recupere
%%  Pid = Identifiant de processus
%%  Cle = Chane de caractre.
%% Renvoie: 
%%  {ok, Valeur}
%%   Valeur = Chane de caractres
%%  {error, not_found}
retrieve(Cle) ->
    ?server ! {retrieve, self(), Cle},
    Self = self(),
    receive
	{helloserver, Self, Cle, not_found} ->
	    {error, not_found};
	{helloserver, Self, Cle, Valeur} ->
	    {ok, Valeur}
    end.

%% Implmentation du serveur lui-mme
%% La boucle loop "stocke" en paramtre les valeurs  mmoriser
loop(Etat) ->
    receive
	{store, Pid, Cle, Valeur} ->
	    NouvelEtat = case lists:keymember(Cle, 1, Etat) of
			     %% Cre l'entre dans la liste
			     false -> Etat ++ [{Cle, Valeur}];
			     %% Remplace l'entre existante
			     true -> lists:keyreplace(Cle, 1, Etat, {Cle, Valeur})
			 end,
	    %% Renvoie la rponse: 
	    Pid ! {helloserver, Pid, Cle},
	    %% Et maintient le serveur
	    loop(NouvelEtat);
	{retrieve, Pid, Cle} ->
	    case lists:keysearch(Cle, 1, Etat) of
		%% La cl n'a pas t trouve:
		false -> Pid ! {helloserver, Pid, Cle, not_found};		     
		%% La cl a t trouve:
		{value, {Cle, Valeur}} -> Pid ! {helloserver, Pid, Cle, Valeur}
	    end,
	    loop(Etat);
	{stop, Pid} ->
	    stopped
    end.
