-module(hello_sup).

-behaviour(supervisor).

%% Les fonctions requises par le comportement supervisor:
-export([init/1]).

%% Les fonctions de contrle du superviseur
%% Seule la fonction start_link va tre implment car un superviseur
%% est toujours dmarrer en informant les processus pre de son arrt.
-export([start_link/0]).

%% La fonction de paramtrage du comportement supervisor
init(_Args) ->
    %% Le cas chant, il est possible d'effectuer des traitements au
    %% dmarrage du serveur. C'est par exemple utile lorsque l'on
    %% souhaite lire le paramtrage du superviseur dans un fichier de
    %% configuration utilisateur.

    %% Le paramtrage se fait dans le formalisme du retour de la fonction:
    %% {ok, {{RestartStrategy, MaxR, MaxT}, [Child1Spec, Child2Spec, ...]}}

    %% Puisque nous n'avons qu'un seul processus supervis, nous
    %% pouvons choisir la stratgie de redmarrage la plus simple:
    %% Seul les processus qui sont en erreur sont redmarrs.
    RestartStrategy = one_for_one,

    %% La gestion des erreurs considrs comme irrmdiable intervient
    %% avec deux paramtres: MaxR et MaxT. Il s'interprte de la
    %% manire suivante: Si MaxR redmarrage surviennent durant un
    %% intervalle de temps de MaxT secondes, alors le superviseur
    %% arrte tous ses processus fils avant de s'arrter
    %% lui-mme. Cela peut dclencher d'autres actions, si notre
    %% superviseur est lui-mme surveill par d'autres processus
    %% superviseur
    MaxR = 3,
    MaxT = 60,
    
    %% Chaque processus supervis est dcrit comme suit:
    %% {ChildName, MFA, Restart, Shutdown, Type, Modules}
    %% Le nom du fils est donn arbitrairement pour dcrire le rle du
    %% processus.

    %% Le tuple {Module, Fonction, Argument} sert  dcrire la manire
    %% dont est lanc un processus donn.

    %% TODO: A dvelopper dans le paragraphe superviseur, la
    %% supervision de processus non conforme au framework Erlang OTP
    %% => Supervisor_bridge.

    %% Le paramtre restart permet de dire si un processus doit tre
    %% redmarr ou pas s'il se termine normalement. Les processus de
    %% type permanent sont toujours redmarrer si les conditions de
    %% frquence de redmarrage sont prsentes.

    %% Le paramtre shutdown permet de dfinir le temps  attendre
    %% pour permettre  un processus de se terminer avec de le
    %% rdemarrer. S'il ne s'est pas arrt normalement une fois le
    %% temps coul (en millisecondes, il est alors arrt
    %% brutalement.

    %% Le type permet de dire au superviseur si le processus dcrit
    %% est un autre superviseur ou un processus travailleur.

    %% le paramtre Modules regroupent la liste des modules entrant
    %% dans la composition du code du processus fils.
    %% Cette information est utilise pour la mise  jour du code.
    Child1Spec = {hellosrv, {hello_srv, start_link, []}, permanent, 10000, worker, [hello_srv]},

    %% Finalement la rponse complte est compos dans le retour de la
    %% fonction init/1
    {ok,{{RestartStrategy, MaxR, MaxT},
	 [
	  Child1Spec
	 ]}}.

%% fonctions de dmarrage du superviseur
start_link() ->
    %% Il existe une fonction supervisor:start_link/3 permettant de
    %% donner un nom au superviseur, mais il est rarement utile de
    %% nommer un processus superviseur (Ce n'est le cas que si l'on
    %% veut contrler prcisment l'arrt ou le dmarrage de ses
    %% processus fils).
    supervisor:start_link(?MODULE, []).
