-module(edb).
-export([list_tables/2, table_schema/2]).

list_tables(Env,[]) ->
    [header(),
     top("Gestion des tables de la base de donnees"),
     action(),
     list_tables(),
     action(),
     footer()].

action() ->
    "<P><A HREF=\"/cgi-bin/erl/edb/table_create\">Create Table</A></P>".

list_tables() ->
    Tables = mnesia:system_info(tables),
    %% Do not show the schema_table:
    Tables2 = lists:delete(schema, Tables),
    %% Format the database table list with its available command association:
    Tables3 = lists:map(fun(Elt) -> [Elt,
				     "<A HREF=\"/cgi-bin/erl/edb/table_schema?" ++ to_string(Elt) ++ "\">View schema</A>",
				     "<A HREF=\"cgi-bin/erl/edb/table_browse?" ++ to_string(Elt) ++ "\">Browse content</A>",
				     "<A HREF=\"cgi-bin/erl/edb/table_drop?" ++ to_string(Elt) ++ "\">Delete</A>"] end,
			Tables2),
    html_table(Tables3).

table_schema(Env, []) ->
     print("Visualisation du schema de table: Aucune table n'a ete precisee.");

%% Argument en entree: table
table_schema(Env, Input) ->
    Table = lists:keysearch TODO httpd:parse_query(Input),
    [header(),
     top("Schema de la table ..."),
     %% Barre de menu/navigation
     io_lib:format("~p~n~p~n",[Env,Var]),
     footer()].

%% TODO: Place in a common module.
html_table(List) ->
    ["<TABLE>",
    html_row(List, []),
     "</TABLE>"].
html_row([], Acc) ->
    Acc;
html_row([H|T], Acc) ->
    html_row(T, Acc ++ "<TR>" ++ html_col(H, []) ++ "</TR>").

html_col([], Acc) ->
    Acc;
html_col([H|T], Acc) ->
    html_col(T, Acc ++ "<TD>" ++ to_string(H) ++ "</TD>").

to_string(Data) when atom(Data) ->
    atom_to_list(Data);
to_string(Data) ->
    Data.

print(String) ->
  [header(),
   top("Print"),
   String++"\n",
   footer()].

header() ->
  header("text/html").
header(MimeType) ->
  "Content-type: " ++ MimeType ++ "\r\n\r\n".

top(Title) ->
  "<HTML>
<HEAD>
<TITLE>" ++ Title ++ "</TITLE>
</HEAD>
<BODY>\n".

footer() ->
  "</BODY>
</HTML>\n".
