%% Dbut du module eval_graphique
-module(eval_graphique).

-export([start/0, init/0]).

start() ->
    spawn(?MODULE, init, []),
    ok.

%% Mise en place de l'interface graphique
init() ->
    %% Lancement du systme d'interface graphique
    Gs = gs:start(),

    %% Cration d'une fentre de 500 pixels de large et 400 pixel de haut.
    Win = gs:create(window, Gs, [{title,"Evaluation Erlang"},
                                 {width,500},{height,400}]),

    gs:create(label,Win,[{label,{text,"Expression Erlang"}},
                         {width,150}]),

    %% L'option keypress = true permet de demander de recevoir
    %%  un vnement sur pression de la touche entre
    gs:create(entry,expression,Win,[{x,10},{y,30},{width,130},
                                    {keypress,true}]),

    gs:create(button,eval,Win,[{width,45},{y,60},{x,10},
                               {label,{text,"Eval"}}]),

    %% Zone d'affichage du rsultat
    gs:create(editor, result, Win, [{x,10}, {y, 90}, {width, 400}, {height, 300},
                                    {wrap, word}, {vscroll,right}]),

    gs:config(Win, {map,true}),
    
    loop([]).


%% Boucle de traitement des vnements de l'application
loop(Variables) ->
    receive
        %% L'utilisateur a cliqu sur le bouton 'eval'
        %% l'expression, on l'value et on excute le
        %% traitement et on affiche le rsultat:
        {gs,eval,click,_Data,_Args} ->
             %% On rcupre l'expression:
             Expr = gs:read(expression, text),
             %% On value l'expression
             {Result, NewVariables} = eval(Expr, Variables),
             %% On affiche le rsultat (aprs avoir effac la zone)
             %% Il faut d'abord autoriser l'criture et l'interdire 
             %% la fin de l'opration, car la zone d'affichage est en
             %% lecture seule
             gs:config(result,{enable, true}),
             gs:config(result, clear),
             gs:config(result, {insert, {insert, Result}}),
             gs:config(result, {enable, false}),
             %% Reste dans l'application
             loop(NewVariables);
        %% On ignore les autres vnements.
        Evenement ->
            loop(Variables)
    end.

%% Evalue l'expression et on intercepte les erreurs pour pouvoir
%% les afficher.
eval(String, Bindings) ->
        case catch eval_expr(String, Bindings) of
           {'EXIT', Erreur} -> Chaine = lists:flatten(io_lib:format("~p", [Erreur])),
                              {Chaine, Bindings};
           {ok, Res, NewBindings} -> {Res, NewBindings}
           end.
eval_expr(String, Bindings) ->
        {ok, Tokens, Endline} = erl_scan:string(String),
        {ok, AbstractForm} = erl_parse:parse_exprs(Tokens),
        {value, Result, NewBindings} = erl_eval:exprs(AbstractForm, Bindings),
        {ok, Result, NewBindings}.
%% Fin du module eval_graphique
