%% Jeu de la vie sur plusieurs cellules

%% Le fonctionnement avec 8 voisins ne marche pas...

-module(cell).
-export([create/2, create/3, loop/7]).
-define(frequence, 1).

% -------------------------------------------------------------------------------------
create (Coordinates, Matrix) ->
    State = case random:uniform(2) of
	       1 -> alive;
	       2 -> dead
	   end,
    create (Coordinates, State, Matrix).
% -------------------------------------------------------------------------------------
create (Coordinates, State, Matrix) ->
    Neighbours = gol:neighbours4 (Coordinates, Matrix),
    DispatcherPid = self(),
    CellPid = spawn (?MODULE, loop,
		     [Coordinates, State, 0, DispatcherPid, Neighbours,
		      [], length(Neighbours)]),

    DispatcherPid ! message (Coordinates, 0, State),
    CellPid.

% -------------------------------------------------------------------------------------
% for text_ui
loop (Coordinates, State, Generation, DispatcherPid,
      Neighbours, MessagesBuffer, 0) ->
    receive % wait for a next_gen event
	{nextgen} ->
	    %% Must be in sync with neighbours4/neighbours8
	    NewState = next_state4 (State, MessagesBuffer),
	    DispatcherPid ! message (Coordinates, Generation + 1, NewState),
	    loop (Coordinates, NewState, Generation + 1, DispatcherPid,
		  Neighbours, [], length (Neighbours));
	{terminate} ->
	    ok
    after 30000 ->
	exit(ok)
    end;

loop (Coordinates = {X,Y}, State, Generation, DispatcherPid,
      Neighbours, MessagesBuffer, MissingNeighbours) ->
    receive
	{cell, CoordCell = {I,J}, Generation, CellState} ->
	    %%io:format ("cellule (~p,~p): message cell recu de (~p,~p)~n",[X,Y,I,J]),
	    case lists:member (CoordCell, Neighbours) of
		true  ->
		    %io:format ("cell (~p,~p): msg recvd from neighbour (~p,~p)~n", [X,Y,I,J]),
		    loop (Coordinates, State, Generation, DispatcherPid,
			  Neighbours, [CellState | MessagesBuffer],
			  MissingNeighbours - 1);

		false ->
		    %io:format ("cell (~p,~p): msg recvd from NON neighbour (~p,~p)~n",[X,Y,I,J]),
		    loop (Coordinates, State, Generation, DispatcherPid,
			  Neighbours, MessagesBuffer, MissingNeighbours)
	    end

    after 5000 -> %io:format ("cell (~p,~p): no messages received for 5 secs, exiting...~n",[X,Y]),
		  exit (ok)
    end.

% ------------------------------------------------------------------------------------
next_state4 (State, NeighbourStates) ->
    MissingNeighboursNb = count (alive, NeighbourStates),
    case MissingNeighboursNb of
	0 -> dead;
	1 -> alive;
	2 -> alive;
	3 -> dead;
	4 -> dead
    end.

% -------------------------------------------------------------------------------------
next_state8 (State, NeighbourStates = [H|T]) ->
    LivingNeighbours = count (alive, NeighbourStates),
    next_state8 (State, LivingNeighbours);

next_state8 (alive, LivingNeighbours) ->
    case LivingNeighbours of
	2 -> alive;
	3 -> alive;
	_ -> dead
    end;

next_state8 (dead, LivingNeighbours) ->
    case LivingNeighbours of
	3 -> alive;
	%4 -> alive;
	_ -> dead
    end.

% -------------------------------------------------------------------------------------
message (Coordinates = {X,Y}, Generation, State) ->
    %io:format ("cell {~p,~p}, sending message~n", [X,Y]),
    {cell, Coordinates, Generation, State}.

% -------------------------------------------------------------------------------------
display_neighbours ([Last]) ->
    io:format ("(~p,~p)~n", tuple_to_list(Last)),
    ok;
display_neighbours ([Current|Others]) ->
    io:format ("(~p,~p) ", tuple_to_list(Current)),
    display_neighbours (Others).

% -------------------------------------------------------------------------------------
count (State, NeighbourStates) ->
    count (State, NeighbourStates, 0).

count (State, [], Nb) ->
    Nb;
count (State, [State|OtherStates], Nb) ->
    count (State, OtherStates, Nb + 1);
count (State, [NeighbourState|NeighbourStates], Nb) ->
    count (State, NeighbourStates, Nb).
% -------------------------------------------------------------------------------------
