%% Jeu de la vie sur plusieurs cellules

%% Point d'entre : 
%% 1> gol:start().
%% ou
%% 1> gol:start(<nombre de lignes>, <nombre de colonnes>, <nombre max de gnrations>)

%% a faire : 
%% La version avec 8 voisins de marche pas...

-module(gol).
-export([start/0, start/4, start/3, neighbours4/2, neighbours8/2, purge/0]).

%% ------------------------------------------------------------------------------------
start () ->
    %%start(3,3,20).
    start(5,7,100).

% start/3
start (Rows,Cols,MaxGen) ->
    {X,Y,Z} = now(),
    random:seed (X,Y,Z),
    UI = gs_ui,
    io:format("Starting Game of Life in Erlang (using ~w)~n", [UI]),
    start (UI, Rows, Cols, MaxGen).

% start/4
start (UIModuleHandler, X, Y, MaxGen) ->
    purge(),
    Matrix = {X,Y},
    AllCells = [ {I,J} || I <- lists:seq (1,X), J <- lists:seq (1,Y) ],
    CellPids = lists:map (fun (Coord) ->
				  cell:create (Coord, Matrix)
			  end,
			  AllCells),

    % Branchement sur le module text_ui.
    UIPid = spawn (UIModuleHandler, start,
		   [length (CellPids), 0, Matrix, self()]),
    dispatch  ([UIPid|CellPids], Matrix, MaxGen).

% -------------------------------------------------------------------------------------
% Purging remaining messages from previous occurences of game...
purge () ->
    %%io:format("purging old messages..."), 
    do_purge (500).
    %%io:format("done~n").

do_purge (Delay) ->
    receive
	{cell, _, 20, _} -> do_purge (Delay);
	{nextgen}	-> do_purge (Delay);
	{terminate}	-> do_purge (Delay)
    after Delay	        -> ok
    end.

% -------------------------------------------------------------------------------------
dispatch (Pids = [ UIPid | CellPids ], Matrix, MaxGen) ->
    receive
	%% Message ->
	{cell, CoordCell, MaxGen, CellState} ->
	    io:format("dispatch: max nb of generations (~p) reached, quitting...", [MaxGen]),
	    ok;
	%%  a message is received from a cell, dispatch it
	{cell, CoordCell = {X,Y}, Generation, CellState} ->
	    % Only dispatch message to cell neighbours...
	    % FIXME : neighbours are calculated each time
	    Neighbours     = neighbours4 (CoordCell, Matrix),
	    NeighboursPids = neighbours_pids (Neighbours, CellPids, Matrix),
	    lists:foreach (
	      fun (Pid) ->
		      Pid ! {cell, CoordCell, Generation, CellState} %Message
	      end,
	      %%Pids
	      [ UIPid | NeighboursPids ]
	     ),
	    dispatch (Pids, Matrix, MaxGen);
	% messages from UI
	{nextgen} ->
	    lists:foreach (fun (Pid) -> Pid ! {nextgen} end,
			   CellPids),
	    dispatch (Pids, Matrix, MaxGen);
	{terminate} ->
	    io:format("dispatcher: `terminate' received from UI, quitting...~n", []),
	    lists:foreach (fun (Pid) -> Pid ! {terminate} end, CellPids),
	    ok

    after 30000 ->
	    io:format("dispatcher: no activity, stopping listening...~n", []),
	    ok
    end.

% -------------------------------------------------------------------------------------
neighbours4 (Coord = {1,1}, Matrix = {I,J}) ->
    [{1,2}, {2,1}];

neighbours4 (Coord = {I,1}, Matrix = {I,J}) ->
    [{I-1,1}, {I,2}];

neighbours4 (Coord = {1,J}, Matrix = {I,J}) ->
    [{1,J-1}, {2,J}];

neighbours4 (Coord = {I,J}, Matrix = {I,J}) ->
    [{I,J-1}, {I-1,J}];

neighbours4 (Coord = {1,Y}, Matrix = {I,J}) ->
    [{1,Y-1}, {1,Y+1}, {2,Y}];

neighbours4 (Coord = {X,1}, Matrix = {I,J}) ->
    [{X - 1,1}, {X+1,1}, {X,2}];

neighbours4 (Coord = {X,J}, Matrix = {I,J}) ->
    [{X - 1,J}, {X+1,J}, {X,J-1}];

neighbours4 (Coord = {I,Y}, Matrix = {I,J}) ->
    [{I - 1,Y}, {I,Y-1}, {I,Y+1}];

neighbours4 (Coord = {X,Y}, Matrix = {I,J}) ->
    [{X,Y-1}, {X,Y+1}, {X-1,Y}, {X+1,Y}].

%%  -----------------------------------------------------------------------------------
neighbours8 (Coord = {1,1}, Matrix = {N,P}) ->
    [       {1,2},
     {2,1}, {2,2}];

neighbours8 (Coord = {N,1}, Matrix = {N,P}) ->
    [{N-1,1},{N-1,2},
             {N,  2}];

neighbours8 (Coord = {1,P}, Matrix = {N,P}) ->
    [{1,P-1},
     {2,P-1}, {2,P}];

neighbours8 (Coord = {N,P}, Matrix = {N,P}) ->
    [{N-1,P-1}, {N-1,P}, 
     {N,  P-1}];

neighbours8 (Coord = {1,J}, Matrix = {N,P}) ->
    [{1,J-1},           {1,J+1},
     {2,J-1},  {2,J},   {2,J+1}];

neighbours8 (Coord = {N,J}, Matrix = {N,P}) ->
    [{1,J-1},  {N-1,J}, {1,J+1},
     {N,J-1},           {N,J+1}];

neighbours8 (Coord = {I,1}, Matrix = {N,P}) ->
    [{I-1,1},  {I-1,2},
               {I,  2},
     {I+1,1},  {I+1,2}];

neighbours8 (Coord = {I,P}, Matrix = {N,P}) ->
    [{I-1,P-1},  {I-1,P},
     {I,  P-1},
     {I+1,P-1},  {I+1,P}];

neighbours8 (Coord = {I,J}, Matrix = {N,P}) ->
    [{I-1,J-1}, {I-1,J}, {I-1,J+1},
     {I  ,J-1},          {I  ,J+1},
     {I+1,J-1}, {I+1,J}, {I+1,J+1}].

% -------------------------------------------------------------------------------------
% calculs
neighbours_pids (Neighbours, AllPids, Dims) ->
    neighbours_pids (Neighbours, AllPids, Dims, []).

%% returns, in order the Pids of Neighbours
neighbours_pids (Neighbours = [{X,Y}], AllPids, Dims = {N,P}, Acc) ->
    Index = (X - 1)*P + Y,
    Pid = lists:nth (Index, AllPids),
    [Pid|Acc];
neighbours_pids (Neighbours = [{X,Y}|Others], AllPids, Dims = {N,P}, Acc) ->
    Index = (X - 1)*P + Y,
    Pid = lists:nth (Index, AllPids),
    neighbours_pids (Others, AllPids, Dims, [Pid|Acc]).
