-module(text_ui).
-compile(export_all).

% DispatcherPid added for compatibility with gs_ui
start (NbCell, 0, Matrix, DispatcherPid) ->
    io:format("Game of Life in Erlang~n", []),
    loop (NbCell, Matrix, 0, NbCell, []).

loop (NbCell, Matrix, Generation, 0, StatesAlist) ->
    io:format("Generation ~p:~n", [Generation]),
    display (Matrix, StatesAlist),
    loop (NbCell, Matrix, Generation + 1, NbCell, []);

loop (NbCell, Matrix, Generation, RemainingCellsNb, StatesAlist) ->
    receive
	{cell, Coord = {X,Y}, Generation, CellState} ->
	    loop (NbCell, Matrix, Generation, RemainingCellsNb - 1,
		  [{Coord, CellState} | StatesAlist])
    after 5000 ->
	    io:format ("text_ui: no messages received for 5 secs, exiting...~n", []),
	    exit(ok)
    end.

display (Matrix = {N,P}, StatesAlist) ->
    AllCells = [ {I,J} || I <- lists:seq(1,N),
			  J <- lists:seq(1,P) ],
    lists:foreach (
      fun (Cell = {I,J}) ->
	      case lists:keysearch (Cell, 1, StatesAlist) of 
		  {value, {Cell, State}} ->
		      io:format("~c", [cell_representation(State)]),
		      case J of
			  P	-> io:format("~n", []);
			  Other	-> ok
		      end;
		  false ->
		      io:format("?", [])
	      end
      end,
      AllCells).

cell_representation (alive) ->    $+;
cell_representation (dead)  ->    $..
